@@ -281,12 +281,12 @@ module Dnsruby
 
     def self.verify(msg, keys=nil)
       begin
-        return true if @@anchor_verifier.verify(msg, keys=nil)
+        return true if @@anchor_verifier.verify(msg, keys)
       rescue VerifyError
         begin
-          return true if @@root_verifier.verify(msg, keys=nil)
+          return true if @@root_verifier.verify(msg, keys)
         rescue VerifyError
-          return true if @@dlv_verifier.verify(msg, keys=nil) # Will carry error to client
+          return true if @@dlv_verifier.verify(msg, keys) # Will carry error to client
         end
       end
     end
