@@ -24,7 +24,7 @@ class TestRecur < Test::Unit::TestCase
 #    Dnsruby::TheLog.level = Logger::DEBUG
     ret = r.query("uk-dnssec.nic.uk", Dnsruby::Types.DNSKEY)
 #    print ret
-    assert(ret.answer.length > 0)
+    assert(ret && ret.answer.length > 0)
 #    ret = r.query_dorecursion("aaa.bigzone.uk-dnssec.nic.uk", Dnsruby::Types.DNSKEY)
 #    ret = r.query_dorecursion("uk-dnssec.nic.uk", Dnsruby::Types.DNSKEY)
   end
